package org.apache.sshd.common.keyprovider;

import java.security.KeyPair;
import java.util.ArrayList;
import java.util.List;

import org.apache.sshd.common.KeyPairProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.apache.sshd.common.util.KeyUtils.getKeyType;

/**
 * TODO Add javadoc
 *
 * @author <a href="mailto:dev@mina.apache.org">Apache MINA SSHD Project</a>
 */
public abstract class AbstractKeyPairProvider implements KeyPairProvider {

    protected final Logger log = LoggerFactory.getLogger(getClass());

    public KeyPair loadKey(String type) {
        assert type != null;
        String realType = type;
        if ("rsa-sha2-256".equals(type) || "rsa-sha2-512".equals(type)) {
            realType = "ssh-rsa";
        }
        Iterable<KeyPair> keys = loadKeys();
        for (KeyPair key : keys) {
            if (realType.equals(getKeyType(key))) {
                return key;
            }
        }
        return null;
    }

    public String getKeyTypes() {
        List<String> types = new ArrayList<String>();
        Iterable<KeyPair> keys = loadKeys();
        for (KeyPair key : keys) {
            String type = getKeyType(key);
            if (type != null && !types.contains(type)) {
                if ("ssh-rsa".equals(type)) {
                    types.add("rsa-sha2-256");
                    types.add("rsa-sha2-512");
                }
                types.add(type);
            }
        }
        StringBuilder sb = new StringBuilder();
        for (String type : types) {
            if (sb.length() > 0) {
                sb.append(",");
            }
            sb.append(type);
        }
        return sb.toString();
    }

    public abstract Iterable<KeyPair> loadKeys();
}
