package org.apache.sshd.common.signature;

import java.security.PrivateKey;
import java.security.PublicKey;

import org.apache.sshd.common.Signature;
import org.apache.sshd.common.util.SecurityUtils;

/**
 * TODO Add javadoc
 * @author <a href="mailto:dev@mina.apache.org">Apache MINA SSHD Project</a>
 */
public abstract class AbstractSignature implements Signature {

    protected java.security.Signature signature;

    protected String algorithm;
    
    protected String sshAlgorithm;

    protected AbstractSignature(String algorithm) {
        this(algorithm, null);
    }
    
    protected AbstractSignature(String algorithm, String sshAlgorithm) {
        this.algorithm = algorithm;
        this.sshAlgorithm = sshAlgorithm;
    }


    public void init(PublicKey pubkey, PrivateKey prvkey) throws Exception {
        signature = SecurityUtils.getSignature(algorithm);
        if (pubkey != null) {
            signature.initVerify(pubkey);
        }
        if (prvkey != null) {
            signature.initSign(prvkey);
        }
    }

    public void update(byte[] foo, int off, int len) throws Exception {
        signature.update(foo, off, len);
    }

    protected byte[] extractSig(byte[] sig) {
        if (sig[0] == 0 && sig[1] == 0 && sig[2] == 0) {
            int i = 0;
            int j;
            j = ((sig[i++] << 24) & 0xff000000) | ((sig[i++] << 16) & 0x00ff0000) | ((sig[i++] << 8) & 0x0000ff00) | ((sig[i++]) & 0x000000ff);
            i += j;
            j = ((sig[i++] << 24) & 0xff000000) | ((sig[i++] << 16) & 0x00ff0000) | ((sig[i++] << 8) & 0x0000ff00) | ((sig[i++]) & 0x000000ff);
            byte[] tmp = new byte[j];
            System.arraycopy(sig, i, tmp, 0, j);
            sig = tmp;
        }
        return sig;
    }

    @Override
    public String getSshAlgorithmName(String algo) {
        if (sshAlgorithm != null) {
            return sshAlgorithm;
        }
        return algo;
    }
}
